/***********************************************************************************************
 * Firesale.cpp
 * Implementation file for the Firesale class.
 * Revised May 1 2010
 *
 * Copyright (C) 2009, Peter Howitt and Quamrul Ashraf, Dept. of Economics, Brown University.
 * All rights reserved.
 ***********************************************************************************************/

#include "Firesale.h"
#include "Agent.h"

Firesale::Firesale(int gdIndx) {
  _gIndex = gdIndx;
}

Firesale::~Firesale(){
}

void Firesale::add_seller(Agent* seller) {
	fQueue.push(seller);
}

void Firesale::drop_sellr(Agent* seller) {
  int nSllrs = static_cast<int> (fQueue.size());
  for(int i = 0; i < nSllrs; i++) {
    if (seller != fQueue.front())
      fQueue.push(fQueue.front());
    fQueue.pop();
  }
}

double Firesale::run_frsale(double fOrder) {
  if (fQueue.size() == 0)
		return 0;
	double unflld = fOrder;
	while (unflld > 0 && fQueue.size() > 0)
    unflld -= fQueue.front()->liquidate(_gIndex, unflld);
	return fOrder - unflld;
}

void Firesale::ini_frsale() {
  while(fQueue.size())
    fQueue.pop();
}

const double Firesale::get_supply() {
  int nSllrs = static_cast<int> (fQueue.size());
  if(nSllrs == 0) return 0;
  double supply = 0;
  for(int i = 0; i < nSllrs; i++) {
    supply += fQueue.front()->get_fOffer(_gIndex);
    fQueue.push(fQueue.front());
    fQueue.pop();
  }
  return supply;
}

Agent* Firesale::dbg_chkDup() {
  int nSllrs = static_cast<int> (fQueue.size());
  if(nSllrs < 2) return NULL;
  for (int i = 0; i < nSllrs; i++) { 
    Agent* nxSllr = fQueue.front();
    fQueue.pop();
    drop_sellr(nxSllr);
    if(static_cast<int>(fQueue.size()) < nSllrs-1) return nxSllr;
    fQueue.push(nxSllr);
  }
  return NULL;
}